.wy-nav-content {
   max-width: 1000px
}
.wy-nav-top,
.wy-side-nav-search {
	background-color: #ff6600;
}

.wy-menu-vertical header, .wy-menu-vertical p.caption{
	color: #ff8533;
}

a{
	color: #cc5200;
}

.rst-content dl:not(.docutils) dt{
	color: #dd7533;
	border-top-color: #ff6600;
	background: #fae7f2 ;
}
